﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Program_4
{
    class GroundPackage
    {
        private int _originZip;
        private int _destinationZip;
        private double _length;
        private double _width;
        private double _height;
        private double _weight;
        public const int DEFAULTORIGINZIP = 40204;
        public const int MINORIGIN = 00000;
        public const int MAXORIGIN = 99999;
        public const int MINDESTINATION = 00000;
        public const int MAXDESTINATION = 99999;
        public const int DEFAULTDESTINATIONZIP = 60606;
        public const int MINLENGTH = 0;
        public const int MINWIDTH = 0;
        public const int MINHEIGHT = 0;
        public const int MINWEIGHT = 0;
        public const int DEFAULTLENGTH = 12;
        public const int DEFAULTWIDTH = 12;
        public const int DEFAULTHEIGHT = 12;
        public const int DEFAULTWEIGHT = 12;
        public const double SIZE_COST_FACTOR = .20;
        public const double WEIGHT_COST_FACTOR = .35;
        public const int DEFAULTDIVISOR = 10000;


        // Precondition:  MINORIGIN < origin zip < MAXORIGIN
        //                MINDESTINATION < destination zip < MAXDESTINATION
        //                MINLENGTH < length
        //                MINWIDTH < width
        //                MINHEIGHT < height
        //                MINWEIGHT < weight
        // Postcondition: The GroundPackage object has been initialized with the specified
        //                originzip, destinationzip, length, width, height, weight 
        public GroundPackage(int originzip, int destinationzip, double length, double width, double height, double weight)
        {
            OriginZip = originzip;//set origin zipcode
            DestinationZip = destinationzip;//set destination zipcode
            Length = length;//set length
            Width = width;//set width
            Height = height;//set height
            Weight = weight;//set weight
        }



        public int OriginZip
        {
            // Precondition:  None
            // Postcondition: The origin zipcode has been returned
            get
            {
                return _originZip;
            }
            // Precondition: 00000 (MINORIGIN) <= value < 99999 (MAXORIGIN)
            // Postcondition: The origin zip has been set to the specified value
            set
            {
                if (value >= MINORIGIN & value <= MAXORIGIN)
                    _originZip = value;
                else //when invalid,set to 40204 instead
                    _originZip = DEFAULTORIGINZIP;
            }
        }

        public int DestinationZip
        {
            // Precondition:  None
            // Postcondition: The destination zipcode has been returned
            get
            {
                return _destinationZip;
            }
            // Precondition: 00000 (MINDESTINATION) <= value < 99999 (MAXDESTINATION)
            // Postcondition: The destination zip has been set to the specified value
            set
            {
                if (value >= MINDESTINATION & value <= MAXDESTINATION)
                    _destinationZip = value;
                else //when invalid,set to 60606 instead
                    _destinationZip = DEFAULTDESTINATIONZIP;
            }
        }



        public double Length
        {
            // Precondition:  None
            // Postcondition: The length has been returned
            get
            {
                return _length;
            }
            // Precondition:  0 (MINLENGTH) < value 
            // Postcondition: The length has been set to the specified value
            set
            {
                if (value > MINLENGTH)
                    _length = value;
                else //when invalid,set to 12 instead
                    _length = DEFAULTLENGTH;

            }
        }
        public double Width
        {
            get
            {
                return _width;
            }
            // Precondition:  0 (MINWIDTH) < value
            // Postcondition: The width has been set to the specified value
            set
            {
                if (value > MINWIDTH)
                    _width = value;
                else //when invalid,set to 12 instead
                    _width = DEFAULTWIDTH;

            }

        }

        public double Height
        {
            get
            {
                return _height;
            }
            // Precondition:  0 (MINHEIGHT) < value 
            // Postcondition: The height has been set to the specified value
            set
            {
                if (value > MINHEIGHT)
                    _height = value;
                else //when invalid,set to 12 instead
                    _height = DEFAULTWIDTH;

            }

        }

        public double Weight
        {
            get
            {
                return _weight;
            }
            // Precondition:  0 (MINWEIGHT) < value
            // Postcondition: The weight has been set to the specified value
            set
            {
                if (value > MINWEIGHT)
                    _weight = value;
                else //when invalid, set to 12 instead
                    _weight = DEFAULTWEIGHT;

            }

        }

        public int ZoneDistance
        {
            // Precondition:  None
            // Postcondition: The zone distance has been returned
            get
            {
                int zonedistance = (OriginZip / DEFAULTDIVISOR) - (DestinationZip / DEFAULTDIVISOR);
                return Math.Abs (zonedistance) ;
            }
        }
        
        public double CalcCost()
        {
            double calccost = SIZE_COST_FACTOR * (Length + Width + Height) + WEIGHT_COST_FACTOR * (ZoneDistance + 1) * (Weight);
            return calccost;
        }
        public override string ToString() //override is required and will output string format 
        {
            return "Origin: " + OriginZip.ToString("D5") + Environment.NewLine + "Destination:" + DestinationZip.ToString("D5") + Environment.NewLine + "Length:" + Length.ToString("N1") + Environment.NewLine + "Width:" + Width.ToString("N1") + Environment.NewLine + "Height:" + Height.ToString("N1") + Environment.NewLine + "Weight:" + Weight.ToString("N1");
        }


    }
}
